'use strict';
const electron = require('electron');
const yaml = require('js-yaml');
const fs = require('fs');
const Parser = require('./lib/Parser.js');
const FileWatcher = require('./lib/FileWatcher.js');
const HttpServer = require('./lib/HttpServer.js');
const path = require('path');
const notifier = require('node-notifier');
const util = require('util');
const moment = require('moment');
const globule = require('globule');

const app = electron.app; // Module to control application life.
const Menu = electron.Menu;
const Tray = electron.Tray;
const dialog = electron.dialog;
const ipcMain = electron.ipcMain;
const BrowserWindow = electron.BrowserWindow; // Module to create native browser window.

// Keep a global reference of the window object, if you don't, the window will
// be closed automatically when the JavaScript object is garbage collected.
let mainWindow;
let config;
let appIcon;
let notify = true;
let watchers = [];

let httpPort = 5364;

let configDir = 'rmconnect';

if (process.env.DEBUG)
    process.env.NODE_TLS_REJECT_UNAUTHORIZED = "0";

function getUserHome() {
    return process.env.HOME || process.env.USERPROFILE;
}

 // Fall thru if it already exists
try { fs.mkdirSync(path.join(getUserHome(), '/'+configDir)); } catch (e) { }

try {
    config = yaml.safeLoad(fs.readFileSync(path.join(getUserHome(), '/'+configDir+'/config.yml'), 'utf8'));
    if (!config)
        config = [];
} catch (e) {
    fs.writeFileSync(path.join(getUserHome(), '/'+configDir+'/config.yml'), "");
    config = [];
}

let logsPath = path.join(getUserHome(), '/'+configDir+'/logs/');

// Fall thru if it already exists
try { fs.mkdirSync(logsPath); } catch (e) {  }
// Fall thru if it already exists
try { fs.mkdirSync(path.join(getUserHome(), '/'+configDir+'/debug')); } catch (e) {  }

cleanLogs();
let logCleanInterval = setInterval(function () {
    cleanLogs();
},24*3600*1000);


const server = new HttpServer(config, watchers, getUserHome(), '/'+configDir);


// Quit when all windows are closed.
app.on('window-all-closed', function() {
    var log = util.format('[%s] %s', new Date().toLocaleString(), "All windows are being closed");
    writeToLog('general_debug', log);
    if (process.platform != 'darwin') {
        app.quit();
    }
});

app.on('quit', function(event, exitCode) {
    var log = util.format('[%s] %s %s', new Date().toLocaleString(), "Process just quit with code: ", exitCode);
    writeToLog('general_debug', log);
})

app.on('ready', function() {
    if (!config) return app.quit();

    var log = util.format('[%s] %s', new Date().toLocaleString(), "Process is starting up");
    writeToLog('general_debug', log);

    mainWindow = new BrowserWindow({
        width: 980,
        height: 580,
        title: "RadioManager Connect",
        icon: path.join(__dirname, 'gfx', 'pluxbox.png'),
        frame: false,
        resizable: true

    });

    mainWindow.loadURL('file://' + __dirname + '/app/index.html');

    // open the dev tools
    // mainWindow.webContents.openDevTools();

    if (process.env.DEBUG)
        mainWindow.hide();

    ipcMain.on('configUpdated', function(event, newConfig) {
        console.log("Config updated, restarting watchers");
        for (var i = 0, l = watchers.length; i < l; i++) {
            watchers[i].kill();
        }
        watchers = [];
        config = newConfig;
        setupWatchers();
        updateTrayMenu();
    });

    ipcMain.on('configUpdateActive', function(event, key, state) {
        if (state){
            setupWatcher(key);
        } else {
            for (var i = 0, l = watchers.length; i < l; i++) {
                if (watchers[i].key == key){
                    watchers[i].kill();
                    watchers.splice(i, 1);
                    console.log("Stopped " + key);
                    break;
                }
            }
        }
    });

    ipcMain.on('setPort', function (event) {
        event.sender.send('setPortReply', httpPort);
    });

    appIcon = new Tray(path.join(__dirname, 'gfx', 'pluxbox-tray.png'));
    appIcon.setToolTip('RadioManager Connect');
    appIcon.on('click', function() {
        mainWindow.show();
    });

    setupWatchers();
    updateTrayMenu();

    var log = util.format('[%s] %s', new Date().toLocaleString(), "Process is now running");
    writeToLog('general_debug', log);

    
});

function sendNotification(status, body) {
    if (!notify) return false;
    notifier.notify({
        'title': 'RadioManager Connect',
        'message': util.format("%s [%s]", this.key, status) // jshint ignore:line
    });
}

function addRequestToLog(status, body) {
    /* jshint validthis: true */
    let log;
    try {
        let b = JSON.parse(body);
        if (b.error)
            log = util.format('[ERR] [%s] %s %s', new Date().toLocaleString(), b.error, b.description);
        else
            log = util.format('[OK] [%s] %s', new Date().toLocaleString(), status);
    } catch (e) {
        log = util.format('[ERR] [%s] %s', new Date().toLocaleString(), body);
    }

    writeToLog(this.key, log);
}

function addErrorToLog(error) {
    /* jshint validthis: true */
    writeToLog(this.key, util.format('[%s] %s', new Date().toLocaleString(), error.message));
}

function writeToLog(key, msg) {
    const date = new moment();
    const cleanKey = key.replace(/(\W|\s)/g, '_');
    
    const filename = logsPath + "/" + cleanKey + "/" + date.format("YYYY-MM-DD") + '.log';

    var dirname = path.dirname(filename);
    if (!fs.existsSync(dirname)) {
        fs.mkdirSync(dirname);
    }
    
    fs.appendFileSync(filename, msg + "\n\r", 'utf-8');
    if (typeof mainWindow != "undefined") {
        mainWindow.webContents.send('logAdded', key, msg);
    }

}

function setupWatchers() {
    for (let key in config) {
        if (config.hasOwnProperty(key)) {
            if (config[key] && config[key].active) {
                setupWatcher(key, true);
            }
        }
    }
    server.update(config, watchers);
}

function setupWatcher(key, multiple = false) {
    if (config[key]) {
        if (
            (config[key].useHttp && config[key].url) ||
            (!config[key].useHttp && config[key].path)
        ) {
            watchers.push(new FileWatcher(config[key], key)
                .on('request', sendNotification)
                .on('request', addRequestToLog)
                .on('error', addErrorToLog));
        }
        if (multiple === false) {
            server.update(config, watchers);
        }
    }
}

function updateTrayMenu() {
    let contextMenu = [{
        enabled: false,
        label: "Watching:",
    }];

    for (let key in config) {
        if (config[key]){
            contextMenu.push({
                label: key,
                submenu: [{
                    label: config[key].endpoint,
                    enabled: false,
                }, {
                    label: config[key].path,
                    enabled: false,
                }]
            });
        }
    }

    contextMenu.push({
        type: 'separator'
    }, {
        label: "Notice on event",
        type: 'checkbox',
        checked: notify,
        click(item) {
            notify = !notify;
        }
    }, {
        label: "Show window",
        click() {
            mainWindow.show();
        }
    }, {
        label: "Quit",
        click() {
            app.quit();
        }
    });

    appIcon.setContextMenu(Menu.buildFromTemplate(contextMenu));
}



function cleanLogs() {
    let allLogs = globule.find(logsPath + '/*/*.log');
    let twoMonths = moment().subtract(3,'months');

    var log = util.format('[%s] %s', new Date().toLocaleString(), "Start cleaning logs");
    writeToLog('general_debug', log);
    
    allLogs.forEach(function(logfile){
        let stat = fs.statSync(logfile);
        let mtime = moment(stat.mtime);
        
        if (mtime < twoMonths) {
            var log = util.format('[%s] %s %s', new Date().toLocaleString(), "Cleaning log: ", logfile);
            writeToLog('general_debug', log);
            fs.unlink(logfile);
        }
    });
    
}

server.runServer(httpPort);